@@ -39,7 +39,6 @@ for my $tranche ( 1 .. 14 ) {
 # SCROLLED SEARCH
 isa_ok $r = $es->scrolled_search(
     q      => 'foo bar',
-    size   => 5,
     scroll => '2m'
     ),
     'ElasticSearch::ScrolledSearch';
@@ -48,9 +47,9 @@ is $r->total, 25, ' - total';
 ok $r->max_score > 0, ' - max_score';
 
 my @docs;
-is scalar( @docs = $r->next ), 5, ' - next()';
-is scalar( @docs = $r->next(1) ),   1,  ' - next(1)';
-is scalar( @docs = $r->next(100) ), 19, ' - next(100)';
+is scalar( @docs = $r->next() ), 1, ' - next()';
+is scalar( @docs = $r->next(2) ),   2,  ' - next(2)';
+is scalar( @docs = $r->next(100) ), 22, ' - next(100)';
 
 ok $r->eof, ' - eof';
 
